import { FIRST_ARRAY_ELEMENT } from '@/constants/common';
import { bioEntitiesContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { MODELS_MOCK } from '@/models/mocks/modelsMock';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { AppDispatch, RootState, StoreType } from '@/redux/store';
import { Accordion } from '@/shared/Accordion';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen } from '@testing-library/react';
import { MockStoreEnhanced } from 'redux-mock-store';
import { BioEntitiesAccordion } from './BioEntitiesAccordion.component';

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <Accordion>
          <BioEntitiesAccordion />
        </Accordion>
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

const renderComponentWithActionListener = (
  initialStoreState: InitialStoreState = {},
): { store: MockStoreEnhanced<Partial<RootState>, AppDispatch> } => {
  const { Wrapper, store } = getReduxStoreWithActionsListener(initialStoreState);

  return (
    render(
      <Wrapper>
        <Accordion>
          <BioEntitiesAccordion />
        </Accordion>
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('BioEntitiesAccordion - component', () => {
  it('should display loading indicator when bioEntity search is pending', () => {
    renderComponent({
      bioEntity: {
        data: [
          {
            searchQueryElement: '',
            loading: 'pending',
            error: { name: '', message: '' },
            data: bioEntitiesContentFixture,
          },
        ],
        loading: 'pending',
        error: { name: '', message: '' },
      },
      models: {
        data: [],
        loading: 'pending',
        error: { name: '', message: '' },
      },
    });

    expect(screen.getByText('Content (Loading...)')).toBeInTheDocument();
  });

  it('should render list of maps with number of entities after succeeded bio entity search', () => {
    renderComponent({
      bioEntity: {
        data: [
          {
            searchQueryElement: '',
            loading: 'succeeded',
            error: { name: '', message: '' },
            data: bioEntitiesContentFixture,
          },
        ],
        loading: 'succeeded',
        error: { name: '', message: '' },
      },
      models: {
        data: MODELS_MOCK,
        loading: 'succeeded',
        error: { name: '', message: '' },
      },
    });

    expect(screen.getByText('Content (10)')).toBeInTheDocument();
    expect(screen.getByText('Core PD map (3)')).toBeInTheDocument();
    expect(screen.getByText('Histamine signaling (4)')).toBeInTheDocument();
    expect(screen.getByText('PRKN substrates (3)')).toBeInTheDocument();
  });

  it('should fire toggleIsContentTabOpened on accordion item button click', () => {
    const { store } = renderComponentWithActionListener({
      ...INITIAL_STORE_STATE_MOCK,
      bioEntity: {
        data: [
          {
            searchQueryElement: '',
            loading: 'succeeded',
            error: { name: '', message: '' },
            data: bioEntitiesContentFixture,
          },
        ],
        loading: 'succeeded',
        error: { name: '', message: '' },
        isContentTabOpened: false,
      },
      models: {
        data: MODELS_MOCK,
        loading: 'succeeded',
        error: { name: '', message: '' },
      },
    });

    const button = screen.getByTestId('accordion-item-button');

    button.click();
    expect(store.getActions()[FIRST_ARRAY_ELEMENT]).toStrictEqual({
      payload: true,
      type: 'bioEntityContents/toggleIsContentTabOpened',
    });
  });
});
