import {
  BACKGROUNDS_MOCK,
  BACKGROUND_INITIAL_STATE_MOCK,
} from '@/redux/backgrounds/background.mock';
import { initialMapStateFixture } from '@/redux/map/map.fixtures';
import { RootState, store } from '@/redux/store';
import { setBackgroundtoEmptyIfAvailable } from './setBackgroundtoEmptyIfAvailable';

const DEFAULT_BACKGROUND_ID = 0;
const EMPTY_BACKGROUND_ID = 15;

describe('setEmptyBackground', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });
  const getStateSpy = jest.spyOn(store, 'getState');
  const dispatchSpy = jest.spyOn(store, 'dispatch');
  it('should not set background to empty if its not available', () => {
    getStateSpy.mockImplementation(
      () =>
        ({
          map: initialMapStateFixture,
          backgrounds: BACKGROUND_INITIAL_STATE_MOCK,
        }) as RootState,
    );

    expect(store.getState().map.data.backgroundId).toBe(DEFAULT_BACKGROUND_ID);

    setBackgroundtoEmptyIfAvailable();

    expect(dispatchSpy).not.toHaveBeenCalled();
  });

  it('should set background to empty if its available', () => {
    getStateSpy.mockImplementation(
      () =>
        ({
          map: initialMapStateFixture,
          backgrounds: { ...BACKGROUND_INITIAL_STATE_MOCK, data: BACKGROUNDS_MOCK },
        }) as RootState,
    );

    expect(store.getState().map.data.backgroundId).toBe(DEFAULT_BACKGROUND_ID);

    setBackgroundtoEmptyIfAvailable();

    expect(dispatchSpy).toHaveBeenCalledWith({
      payload: EMPTY_BACKGROUND_ID,
      type: 'map/setMapBackground',
    });
  });
});
